==> cf-variable-resolver-0.0.10.tgz: package/lib/clients/hashicorp-vault.js <==
const { readFile } = require('fs').promises;
const { URL, URLSearchParams } = require('url');
const _ = require('lodash');
const fetch = require('node-fetch');
const vault = require('node-vault');
const CFError = require('cf-errors');
const { buildError, to } = require('../utils');
const { Type: { InitError, AuthenticationError, MissingReferenceError } } = require('../errors');

const DEFAULT_OPTIONS = {
    apiVersion: 'v1',
    requestOptions: {
        followAllRedirects: true,
        // TODO: add strict ssl indicator
    }
};
const GCE_TOKEN_ENDPOINT = 'http://metadata/computeMetadata/v1/instance/service-accounts/default/identity';
const K8S_TOKEN_PATH = '/var/run/secrets/kubernetes.io/serviceaccount/token';

class Vault {
    constructor(apiUrl, auth) {
        if (!apiUrl) {
            throw buildError(InitError, null, 'Missing apiUrl value for Vault');
        }

        if (_.isEmpty(auth)) {
            throw buildError(AuthenticationError, null, 'Missing auth information for Vault');
        }

        this._apiUrl = apiUrl;
        this._auth = auth;
    }

    /**
     * Initializes the vault client
     */
    async init() {
        const { _auth: auth } = this;
        const { token, type } = auth;
        this._client = vault({
            ...DEFAULT_OPTIONS,
            endpoint: this._apiUrl,
            ...(token && { token }),
        });
        let testFunc;
        switch (type) {

            case 'gcp':
                testFunc = this._gcpLogin.bind(this);
                break;
            case 'kubernetes':
                testFunc = this._kubernetesLogin.bind(this);
                break;
            case 'token':
                testFunc = this._client.tokenLookupSelf.bind(this.client);
                break;
            default:
                if (this._client[`${type}Login`]) {
                    testFunc = this._client[`${type}Login`].bind(this._client, auth);
                } else {
                    throw buildError(AuthenticationError, null, `Auth method '${type}' is not supported`);
                }
        }

        const [err] = await to(testFunc());
        if (err) {
            throw buildError(AuthenticationError, err, 'Authentication failed');
        }
    }

    async read(path) {
        const [err, res] = await to(this._client.read(path));
        if (err) {
            const { errors, warnings } = _.get(err, 'response.body', {});
            const error = errors && errors[0];
            const warning = warnings && warnings[0];

            if (error || warning) {
                throw new CFError({
                    message: `Failed to read secret with: "${error || warning}".`,
                    statusCode: _.get(err, 'response.statusCode'),
                });
            }

            throw buildError(MissingReferenceError, err);
        }
        return res;
    }

    async _gcpLogin() {
        const { _auth: auth } = this;
        const { roleType, role } = auth;
        if (roleType !== 'gce') {
            throw new CFError(`GCE Role Type '${roleType}' is not supported`);
        }

        const jwt = await this._request(GCE_TOKEN_ENDPOINT, {
            headers: { 'Metadata-Flavor': 'Google' },
            params: {
                audience: `http://vault/${role}`,
                format: 'full',
            },
        });
        await this._client.gcpLogin({ ...auth, jwt });
    }

    async _request(url, { headers, params }) {
        const fullUrl = new URL(url);
        fullUrl.search = new URLSearchParams(params).toString();
        const res = await fetch(fullUrl.toString(), { headers });
        const data = await (res.headers.get('content-type') === 'application/json' ? res.json() : res.text());
        if (!res.ok) {
            throw new CFError({
                message: data,
                status: res.status,
                statusText: res.statusText
            });
        }

        return data;
    }

    async _kubernetesLogin() {
        const { _auth: auth } = this;
        const jwt = auth.jwt || await readFile(K8S_TOKEN_PATH, 'utf8');
        await this._client.kubernetesLogin({ ...auth, jwt });
    }

    /**
     * secret reference
     * @typedef {Object} SecretReference
     * @property {string} path
     * @property {string} key
     */

    /**
     * finds the path and key from a context and a variable string
     * @param {any} ctx vault secret-store context
     * @param {string | undefined} variable variable string
     * @returns {SecretReference} secret reference
     */
    static getReferences(ctx, variable = '') {
        let path = _.get(ctx, 'spec.data.path');
        let key = '';
        if (path && variable.includes('@')) {
            const suggestion = 'The specified context is locked to a specific path and cannot be use to resolve secrets from a different path';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        if (!variable.includes('@')) {
            const suggestion = 'The specified context is not locked to a specific, the secret reference should be using the format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        ([path, key] = variable.split('@'));
        if (!path || !key) {
            const suggestion = 'The reference is missing a path to the specified secret or a secret key, make sure you use the correct format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        return {
            path,
            key,
        };
    }
}

module.exports = Vault;

==> cf-variable-resolver-0.0.16.tgz: package/lib/clients/hashicorp-vault.js <==
const { readFile } = require('fs').promises;
const { URL, URLSearchParams } = require('url');
const _ = require('lodash');
const fetch = require('node-fetch');
const vault = require('node-vault');
const CFError = require('cf-errors');
const { buildError, to, isJsonResponse } = require('../utils');
const { Type: { InitError, AuthenticationError, MissingReferenceError } } = require('../errors');

const DEFAULT_OPTIONS = {
    apiVersion: 'v1',
    requestOptions: {
        followAllRedirects: true,
        // TODO: add strict ssl indicator
    }
};
const GCE_TOKEN_ENDPOINT = 'http://metadata/computeMetadata/v1/instance/service-accounts/default/identity';
const K8S_TOKEN_PATH = '/var/run/secrets/kubernetes.io/serviceaccount/token';

class Vault {
    constructor(apiUrl, auth) {
        if (!apiUrl) {
            throw buildError(InitError, null, 'Missing apiUrl value for Vault');
        }

        if (_.isEmpty(auth)) {
            throw buildError(AuthenticationError, null, 'Missing auth information for Vault');
        }

        this._apiUrl = apiUrl;
        this._auth = auth;
    }

    /**
     * Initializes the vault client
     */
    async init() {
        const { _auth: auth } = this;
        const { token, type } = auth;
        this._client = vault({
            ...DEFAULT_OPTIONS,
            endpoint: this._apiUrl,
            ...(token && { token }),
        });
        let testFunc;
        switch (type) {

            case 'gcp':
                testFunc = this._gcpLogin.bind(this);
                break;
            case 'kubernetes':
                testFunc = this._kubernetesLogin.bind(this);
                break;
            case 'token':
                testFunc = this._client.tokenLookupSelf.bind(this.client);
                break;
            default:
                if (this._client[`${type}Login`]) {
                    testFunc = this._client[`${type}Login`].bind(this._client, auth);
                } else {
                    throw buildError(AuthenticationError, null, `Auth method '${type}' is not supported`);
                }
        }

        const [err] = await to(testFunc());
        if (err) {
            throw buildError(AuthenticationError, err, 'Authentication failed');
        }
    }

    async read(path) {
        const [err, res] = await to(this._client.read(path));
        if (err) {
            const { errors, warnings } = _.get(err, 'response.body', {});
            const error = errors && errors[0];
            const warning = warnings && warnings[0];

            if (error || warning) {
                throw new CFError({
                    message: `Failed to read secret with: "${error || warning}".`,
                    statusCode: _.get(err, 'response.statusCode'),
                });
            }

            throw buildError(MissingReferenceError, err);
        }
        return res;
    }

    async _gcpLogin() {
        const { _auth: auth } = this;
        const { roleType, role } = auth;
        if (roleType !== 'gce') {
            throw new CFError(`GCE Role Type '${roleType}' is not supported`);
        }

        const jwt = await this._request(GCE_TOKEN_ENDPOINT, {
            headers: { 'Metadata-Flavor': 'Google' },
            params: {
                audience: `http://vault/${role}`,
                format: 'full',
            },
        });
        await this._client.gcpLogin({ ...auth, jwt });
    }

    async _request(url, { headers, params }) {
        const fullUrl = new URL(url);
        fullUrl.search = new URLSearchParams(params).toString();
        const res = await fetch(fullUrl.toString(), { headers });
        const data = await (isJsonResponse(res) ? res.json() : res.text());
        if (!res.ok) {
            throw new CFError({
                message: data,
                status: res.status,
                statusText: res.statusText
            });
        }

        return data;
    }

    async _kubernetesLogin() {
        const { _auth: auth } = this;
        const jwt = auth.jwt || await readFile(K8S_TOKEN_PATH, 'utf8');
        await this._client.kubernetesLogin({ ...auth, jwt });
    }

    /**
     * secret reference
     * @typedef {Object} SecretReference
     * @property {string} path
     * @property {string} key
     */

    /**
     * finds the path and key from a context and a variable string
     * @param {any} ctx vault secret-store context
     * @param {string | undefined} variable variable string
     * @returns {SecretReference} secret reference
     */
    static getReferences(ctx, variable = '') {
        let path = _.get(ctx, 'spec.data.path');
        let key = '';
        if (path && variable.includes('@')) {
            const suggestion = 'The specified context is locked to a specific path and cannot be use to resolve secrets from a different path';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        if (!variable.includes('@')) {
            const suggestion = 'The specified context is not locked to a specific, the secret reference should be using the format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        ([path, key] = variable.split('@'));
        if (!path || !key) {
            const suggestion = 'The reference is missing a path to the specified secret or a secret key, make sure you use the correct format: "/path/to/secret@key-name"';
            throw buildError(MissingReferenceError, null, suggestion);
        }

        return {
            path,
            key,
        };
    }
}

module.exports = Vault;
